(function ($, window, document, undefined) {
    "use strict";

    // Create the defaults once
    var pluginName = "YWSFD_Bar",
        queryString = '';


    $(document).ready(function () {
        $('.ywsfd-wrapper').YWSFD_Bar();
    });

    $(document).on('qv_loader_stop', function () {
        $('.ywsfd-wrapper').YWSFD_Bar();
    });

    $(document).on('woocommerce_variation_has_changed', function () {
        load_variations();
    });

    $(document).on('show_variation', function () {
        load_variations();
    });

    function load_variations() {

        var body = $('body');

        if (!body.hasClass('single-product')) {
            return;
        }

        var params = [],
            active_variation = parseInt($('.single_variation_wrap .variation_id, .single_variation_wrap input[name="variation_id"]').val()),
            container = $('#YWSFD_wrapper_' + ywsfd.post_id);

        if (!isNaN(active_variation)) {
            $('.ywsfd-choose-variation').hide();
        } else {
            $('.ywsfd-choose-variation').show();
            $('.ywsfd-just-shared').hide();
            container.hide();
            return;
        }

        if (ywsfd.onsale_variations.length > 0) {

            if ($.inArray(active_variation, ywsfd.onsale_variations) === -1 && !isNaN(active_variation)) {
                container.show();
            } else {
                container.hide();
            }

        }

        if (ywsfd.shared_variations.length > 0) {

            if ($.inArray(active_variation, ywsfd.shared_variations) === -1 && !isNaN(active_variation)) {
                container.show();
                $('.ywsfd-just-shared').hide();
            } else {
                container.hide();
                $('.ywsfd-just-shared').show();
            }

        } else {
            container.show();
        }

        if (ywsfd.custom_url === 'no') {

            $('.variations select').each(function () {
                if ($('option:selected', this).val()) {
                    params.push('attribute_' + $(this).attr('id') + '=' + $('option:selected', this).val());
                }
            });

            queryString = params.join('&');

            body.trigger('facebook_share').trigger('twitter_button').trigger('linkedin_button');

            if (ywsfd.email === 'yes') {
                if (queryString !== '') {
                    container.find('input[name=ywsfd_sharing_url]').val(ywsfd.sharing.url + '?' + queryString);
                }
            }

            if (!isNaN(active_variation)) {

                container.attr('data-variation_id', active_variation);

            }

        }
    }

    // The actual plugin constructor
    function Plugin(element) {
        this.element = element;
        this._name = pluginName;
        this.init();
    }

    // Avoid Plugin.prototype conflicts
    $.extend(Plugin.prototype, {
        /** Load Share Bar*/
        init                : function () {

            if (this.container_processing()) {
                return false;
            }

            if (!this.element.length) {
                return false;
            }

            var self = this;

            $.ajax({
                type    : 'POST',
                url     : ywsfd.ajax_load_url,
                data    : {
                    post_id: this.element.data('id')
                },
                success : function (response) {
                    self.element.html(response);
                    /**
                     * If Facebook active
                     */
                    if (ywsfd.facebook === 'yes') {
                        self.element.find("a.ywsfd-facebook-share").click(function (e) {

                            e.preventDefault();

                            var url = $(this).data('href');

                            FB.ui({
                                method : 'feed',
                                link   : url,
                                caption: '',
                                display: 'popup'
                            }, function (response) {
                                if (typeof response !== 'undefined') {
                                    self.get_coupon('fbshare');
                                }
                            });

                        });
                    }

                    /**
                     * If Twitter active
                     */
                    if (ywsfd.twitter === 'yes') {

                        var tweet_textarea = self.element.find('.ywsfd-twitter-form-wrapper .input-textarea');

                        self.twitter_length(tweet_textarea);
                        tweet_textarea.on('change keyup input', function () {
                            self.twitter_length($(this));
                        });
                        tweet_textarea.on('blur input change', function () {

                            var $this = $(this),
                                form_row = $this.closest('.form-row'),
                                validated = true;

                            if (form_row.is('.ywsfd-validate-required')) {
                                if ($this.val() === '') {
                                    form_row.removeClass('woocommerce-validated').addClass('woocommerce-invalid').addClass('woocommerce-invalid-required-field');
                                    validated = false;
                                }
                            }

                            if (validated) {
                                form_row.addClass('woocommerce-validated').removeClass('woocommerce-invalid').removeClass('woocommerce-invalid-required-field');
                            }

                        });

                        self.element.find('.ywsfd-tweet-button').unbind().bind('click', function (e) {
                            e.preventDefault();
                            $.when(self.twitter_auth()).then(function (response) {
                                self.twitter_show_form(response.oauth_token, response.oauth_verifier)
                            })
                        });

                        $('body').trigger('twitter_button');

                    }

                    /**
                     * If Linkedin active
                     */
                    if (ywsfd.linkedin === 'yes') {

                        self.element.find('.ywsfd-linkedin-button').unbind().bind('click', function (e) {

                            e.preventDefault();

                            $.when(self.linkedin_auth()).then(function (response) {
                                self.linkedin_show_form(response.code, response.state)
                            })

                        });

                        $('body').trigger('linkedin_button');

                    }

                    /**
                     * If Email active
                     */
                    if (ywsfd.email === 'yes') {

                        var form = self.element.find('.ywsfd-email-form-wrapper');

                        form.on('blur input change', '.input-text', function () {
                            var $this = $(this),
                                form_row = $this.closest('.form-row'),
                                validated = true;

                            if (form_row.is('.ywsfd-validate-required')) {
                                if ($this.val() === '') {
                                    form_row.removeClass('woocommerce-validated').addClass('woocommerce-invalid').addClass('woocommerce-invalid-required-field');
                                    validated = false;
                                }
                            }

                            if (form_row.is('.ywsfd-validate-email')) {
                                if ($this.val()) {

                                    /* https://stackoverflow.com/questions/2855865/jquery-validate-e-mail-address-regex */
                                    var pattern = new RegExp(/^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i);

                                    if (!pattern.test($this.val())) {
                                        form_row.removeClass('woocommerce-validated').addClass('woocommerce-invalid').addClass('woocommerce-invalid-required-field');
                                        validated = false;
                                    }
                                }
                            }

                            if (validated) {
                                form_row.addClass('woocommerce-validated').removeClass('woocommerce-invalid').removeClass('woocommerce-invalid-required-field');
                            }
                        });

                        self.element.find('.ywsfd-send-btn').click(function () {
                            form.addClass('opened');

                            if (self.container_processing()) {
                                return false;
                            }

                            $.ajax({
                                type    : 'POST',
                                url     : ywsfd.ajax_email_url,
                                data    : {
                                    ywsfd_friend_email: form.find('.ywsfd-friend-email').val(),
                                    ywsfd_user_email  : form.find('.ywsfd-user-email').val(),
                                    ywsfd_message     : form.find('.ywsfd-message').val(),
                                    ywsfd_sharing_url : form.find('input[name=ywsfd_sharing_url]').val(),
                                    ywsfd_post_id     : self.element.data('id'),
                                    ywsfd_variation_id: self.element.data('variation_id') || 0
                                },
                                success : function (code) {

                                    var result = self.parse_response(code);

                                    try {

                                        if (result.status === 'success') {
                                            self.after_share_redirect(result.redirect);
                                        } else if (result.status === 'failure') {
                                            self.print_error_messages(result, code)
                                        }

                                    } catch (err) {
                                        self.print_error_messages(result, code);
                                    }
                                },
                                dataType: 'html'
                            });

                            return false;
                        });

                        self.element.find('.ywsfd-email').click(function () {
                            self.element.find('.ywsfd-email-form-wrapper').toggleClass('opened')
                        });

                    }

                    if (ywsfd.custom_url === 'no') {

                        $('body')
                            .bind('twitter_button', self.alter_twitter_button)
                            .bind('linkedin_button', self.alter_linkedin_button)
                            .bind('facebook_share', self.alter_facebook_share);

                    }

                    if (ywsfd.onsale_variations.length > 0) {

                        var active_variation = parseInt($('.single_variation_wrap .variation_id, .single_variation_wrap input[name="variation_id"]').val());

                        if ($.inArray(active_variation, ywsfd.onsale_variations) !== -1) {

                            $('.ywsfd-wrapper').hide();

                        }

                    }

                    self.element.removeClass('processing').unblock();

                },
                complete: function () {
                    if (ywsfd.facebook === 'yes') {


                        self.loadScript("//connect.facebook.net/" + ywsfd.locale + "/all.js",
                            function () {
                                FB.init({
                                    appId  : ywsfd.fb_app_id,
                                    xfbml  : true,
                                    version: 'v2.12'
                                });
                                $('body').trigger('facebook_share');
                            });


                    }
                },
                dataType: 'json'
            });


            return false;

        },
        /** Container processing handler*/
        container_processing: function () {

            if (this.element.is('.processing')) {
                return true;
            }

            this.element.addClass('processing');
            this.element.block({
                message   : null,
                overlayCSS: {
                    background: '#fff',
                    opacity   : 0.6
                }
            });

            return false;
        },

        /** Querystring Functions*/
        alter_twitter_button: function () {

            if (queryString !== '') {

                var container = $('#YWSFD_wrapper_' + ywsfd.post_id);

                container.find('.ywsfd-tweet-button').attr('data-href', ywsfd.sharing.url + "?" + queryString);
                container.find('.ywsfd-tweet-text').val(ywsfd.sharing.message + ' - ' + ywsfd.sharing.url + "?" + queryString + ywsfd.sharing.twitter_username);

            }

        },

        alter_linkedin_button: function () {

            if (queryString !== '') {

                var container = $('#YWSFD_wrapper_' + ywsfd.post_id);

                container.find('.ywsfd-linkedin-button').attr('data-href', ywsfd.sharing.url + "?" + queryString);
                container.find('.ywsfd-linkedin-text').val(ywsfd.sharing.message + ' - ' + ywsfd.sharing.url + "?" + queryString);

            }

        },

        alter_facebook_share: function () {

            if (queryString !== '') {
                $('#YWSFD_wrapper_' + ywsfd.post_id).find('.ywsfd-facebook-share').attr('data-href', ywsfd.sharing.url + "?" + queryString);
            }

        },

        loadScript: function (stUrl, fnCallback) {

            var script = document.createElement("script");
            script.type = "text/javascript";

            if (script.readyState) {
                script.onreadystatechange = function () {
                    if (script.readyState === "loaded"
                        || script.readyState === "complete") {
                        script.onreadystatechange = null;
                        fnCallback();
                    }
                };
            }
            else {
                script.onload = function () {
                    fnCallback();
                };
            }

            var elementExists = document.getElementById('facebook-jssdk');

            if (elementExists && !ywsfd.force_fb_reload) {
                return;
            }

            script.src = stUrl;
            script.id = 'facebook-jssdk';
            script.async = true;
            document.getElementsByTagName('head')[0].appendChild(script);
        },

        /** Get the coupon and add it to cart */
        get_coupon    : function (social) {

            if (social !== 'twitter' && social !== 'linkedin') {
                if (this.container_processing()) {
                    return false;
                }
            }

            var self = this;

            $.ajax({
                type    : 'POST',
                url     : ywsfd.ajax_social_url,
                data    : {
                    post_id     : this.element.data('id'),
                    variation_id: self.element.data('variation_id') || 0
                },
                success : function (code) {

                    var result = self.parse_response(code);

                    try {
                        if (result.status === 'success') {
                            self.after_share_redirect(result.redirect);
                        } else if (result.status === 'failure') {
                            self.print_error_messages(result, code);
                        }
                    } catch (err) {
                        self.print_error_messages(result, code);
                    }
                },
                dataType: 'html'
            });

            return false;

        },
        /** Parse AJAX response */
        parse_response: function (response) {
            // Get the valid JSON only from the returned string
            if (response.indexOf('<!--WC_START-->') >= 0) {
                response = response.split('<!--WC_START-->')[1]; // Strip off before after WC_START
            }

            if (response.indexOf('<!--WC_END-->') >= 0) {
                response = response.split('<!--WC_END-->')[0]; // Strip off anything after WC_END
            }

            // Parse
            return $.parseJSON(response);
        },

        /** Manage error message*/
        print_error_messages: function (result, code) {

            // Remove old errors
            this.element.find('.woocommerce-error, .woocommerce-message').remove();

            // Add new errors
            if (result.messages) {
                this.element.find('.ywsfd-errors').append(result.messages);
            } else {
                this.element.find('.ywsfd-errors').append(code);
            }

            // Cancel processing
            this.element.removeClass('processing').unblock();

            // Scroll to top
            $('html, body').animate({
                scrollTop: (this.element.offset().top - 100)
            }, 1000);

        },

        /** Redirection after sharing*/
        after_share_redirect: function (redirect) {

            setTimeout(function () {

                if (ywsfd.redirect) {

                    if (redirect.indexOf("https://") !== -1 || redirect.indexOf("http://") !== -1) {
                        window.location = redirect;
                    } else {
                        window.location = decodeURI(redirect);
                    }

                } else {

                    window.location.reload();

                }

            }, 10000);

        },

        /** Twitter Button Functions */
        twitter_auth      : function () {

            var dfd = $.Deferred(),
                popup = window.open(ywsfd.twitter_login, "", "toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=800,height=450");

            window.twitter_callback = function (oauth_token, oauth_verifier) {
                popup.close();
                dfd.resolve({
                    oauth_token   : oauth_token || null,
                    oauth_verifier: oauth_verifier || null
                });
            };

            return dfd.promise();

        },
        /** Twitter Form Functions */
        twitter_show_form : function (oauth_token, oauth_verifier) {

            var self = this;

            this.element.find('.ywsfd-twitter-form-wrapper').addClass('opened');
            this.element.find('.ywsfd-send-tweet').unbind().bind('click', function () {

                if (self.container_processing()) {
                    return false;
                }

                $.ajax({
                    type    : 'POST',
                    url     : ywsfd.twitter_send_ajax,
                    data    : {
                        oauth_token   : oauth_token || null,
                        oauth_verifier: oauth_verifier || null,
                        tweet         : self.element.find('.ywsfd-tweet-text').val(),
                        sharing_url   : self.element.find('.ywsfd-tweet-button').data('href')
                    },
                    success : function (code) {
                        var result = self.parse_response(code);

                        try {
                            if (result.status === 'success') {
                                self.get_coupon('twitter');
                            } else if (result.status === 'failure') {
                                self.print_error_messages(result, code);
                            }
                        } catch (err) {
                            self.print_error_messages(result, code);
                        }
                    },
                    dataType: 'html'
                });

            });

            this.element.find('.ywsfd-cancel-tweet').unbind().bind('click', function () {
                self.element.find('.ywsfd-twitter-form-wrapper').removeClass('opened');
            });

        },
        /** Twitter text length */
        twitter_length    : function (textarea) {

            if (textarea.val() === undefined) {
                return;
            }

            var length = textarea.val().length,
                chars = 280 - length,
                char_count = this.element.find('.ywsfd-char-count');

            char_count.find('span').text(chars);

            if (chars < 0) {
                char_count.addClass('maxed-out');
            } else {
                char_count.removeClass('maxed-out');
            }

        },
        /** LinkedIn Button Functions */
        linkedin_auth     : function () {

            var dfd = $.Deferred(),
                popup = window.open(ywsfd.linkedin_login, "", "toolbar=0,scrollbars=1,location=0,statusbar=0,menubar=0,resizable=1,width=800,height=450");

            window.linkedin_callback = function (code, state) {
                popup.close();
                dfd.resolve({
                    code : code || null,
                    state: state || null
                });
            };

            return dfd.promise();

        },
        /** LinkedIn Form Functions */
        linkedin_show_form: function (code, state) {

            var self = this;

            this.element.find('.ywsfd-linkedin-form-wrapper').addClass('opened');
            this.element.find('.ywsfd-send-linkedin').unbind().bind('click', function () {

                if (self.container_processing()) {
                    return false;
                }

                $.ajax({
                    type    : 'POST',
                    url     : ywsfd.linkedin_send_ajax,
                    data    : {
                        code       : code || null,
                        state      : state || null,
                        message    : self.element.find('.ywsfd-linkedin-text').val(),
                        sharing_url: self.element.find('.ywsfd-linkedin-button').data('href')
                    },
                    success : function (code) {

                        var result = self.parse_response(code);

                        try {
                            if (result.status === 'success') {
                                self.get_coupon('linkedin');
                            } else if (result.status === 'failure') {
                                self.print_error_messages(result, code);
                            }
                        } catch (err) {
                            self.print_error_messages(result, code);
                        }

                    },
                    dataType: 'html'
                });

            });

            this.element.find('.ywsfd-cancel-linkedin').unbind().bind('click', function () {
                self.element.find('.ywsfd-linkedin-form-wrapper').removeClass('opened');
            });

        }

    });

    // A really lightweight plugin wrapper around the constructor,
    // preventing against multiple instantiations
    $.fn[pluginName] = function () {
        return this.each(function () {
            if (!$.data(this, "plugin_" + pluginName)) {
                $.data(this, "plugin_" +
                    pluginName, new Plugin($(this)));
            }
        });
    };

})(jQuery, window, document);

